' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports System.Globalization
Imports System.ServiceModel
Imports DiscoveryProxy.Contracts
Imports Microsoft.Samples.Discovery.Contracts

Namespace Microsoft.Samples.Discovery.LegacyChat

    Partial Public Class LegacyChatForm
        Inherits Form
        Private cmdIndex As Integer
        Private appState As TerminalState
        Private prompt As String = ">"
        Private userName As String
        Private _users As New UserCollection()
        Private proxy As ILegacyChatProxy
        Private localAddress As Uri
        Private chatServiceHost As ServiceHost

        Public Sub New()
            InitializeComponent()
            Me.AppendNewLine()
            Me.ShowPrompt()
        End Sub

        Private Sub OpenChatService()
            Me.localAddress = New Uri("http://localhost:8000/" & Guid.NewGuid().ToString())

            ' Create a singleton instance for the host
            Me.chatServiceHost = New ServiceHost(New ChatService(Me), Me.localAddress)
            Me.chatServiceHost.AddServiceEndpoint(GetType(ISimpleChatService), New BasicHttpBinding(), Me.localAddress)
            Me.chatServiceHost.Open()
        End Sub

        Private Sub CloseChatService()
            Me.chatServiceHost.BeginClose(TimeSpan.FromSeconds(5), Sub(result) chatServiceHost.EndClose(result), Nothing)
        End Sub

        Friend ReadOnly Property Users As UserCollection
            Get
                Return Me._users
            End Get
        End Property

        Private Sub ShowPrompt()
            Me.TerminalWrite(Me.prompt)
        End Sub

        Private Sub AppendNewLine()
            textBoxTerminal.Text += Environment.NewLine
        End Sub

        Private Sub TerminalWriteLine(ByVal text As String)
            textBoxTerminal.Text += text & Environment.NewLine
            Me.SetCaret()
        End Sub

        Private Sub TerminalWrite(ByVal text As String)
            textBoxTerminal.Text += text
            Me.SetCaret()
        End Sub

        Private Sub SetCaret()
            Me.cmdIndex = textBoxTerminal.Text.Length
            textBoxTerminal.SelectionStart = textBoxTerminal.Text.Length
            textBoxTerminal.ScrollToCaret()
        End Sub

        Private Sub TextBoxTerminal_KeyUp(ByVal sender As Object, ByVal e As KeyEventArgs) Handles textBoxTerminal.KeyUp
            Select Case e.KeyCode
                Case Keys.Enter
                        Select Case Me.appState
                            Case TerminalState.Command
                                Me.ProcessCommand()
                            Case TerminalState.Login
                                Me.ProcessLogin()
                            Case TerminalState.Chat
                                Me.ProcessChat()
                        End Select

                        Exit Select

                Case Keys.Up
                    e.SuppressKeyPress = True
                Case Keys.Down
                    e.SuppressKeyPress = True
            End Select
        End Sub

        Private ReadOnly Property TerminalLine() As Integer
            Get
                Return textBoxTerminal.GetLineFromCharIndex(textBoxTerminal.SelectionStart)
            End Get
        End Property

        Private ReadOnly Property TerminalColumn() As Integer
            Get
                Return textBoxTerminal.SelectionStart - textBoxTerminal.GetFirstCharIndexOfCurrentLine() + 1
            End Get
        End Property

        Private Sub ProcessChat()
            Dim input = Me.TerminalReadLine()
            Dim args As String
            Dim cmd As String = ParseCmd(input, args)
            If (Not String.IsNullOrEmpty(cmd)) Then
                Select Case cmd.ToUpperInvariant()
                    Case "BYE"
                        Me.GoOffline()
                    Case "USERS"
                        Me.ShowUsers()
                    Case "SAY"
                        Me.SayTo(args)
                    Case "QUIT"
                        Me.QuitApp()
                    Case Else
                        Me.TerminalWriteLine(Resources.UserQuit)
                End Select
            End If

            Me.ShowPrompt()
        End Sub

        Private Sub GoOnline()
            Me.TerminalWriteLine(Resources.ChatOnlineAt)
            Me.OpenChatService()
            If Me.proxy Is Nothing Then
                Dim binding As New NetTcpBinding()
                Dim address As New EndpointAddress("net.tcp://localhost:8001/legacyproxy")
                Me.proxy = ChannelFactory(Of ILegacyChatProxy).CreateChannel(binding, address)
            End If

            Me.proxy.Register(Me.localAddress, Me.userName)
            Me.TerminalWriteLine(String.Format(CultureInfo.CurrentCulture, Resources.ChatOnlineAt, Me.localAddress.ToString()))
            Me.ShowUsers()
        End Sub

        Private Sub GoOffline()
            Me.TerminalWriteLine(Resources.GoingOffline)
            If Me.proxy IsNot Nothing Then Me.proxy.Unregister(Me.localAddress)

            Me.CloseChatService()
            Me.TerminalWriteLine(Resources.LoggedOut)
            Me.SetAppState(TerminalState.Command)
        End Sub

        Private Sub SayTo(ByVal args As String)
            Dim split() = args.Split(" "c)
            If split.Length < 2 Then
                Me.TerminalWriteLine(Resources.ErrorNoMessage)
                Return
            End If

            Dim user = split(0)
            If Me._users.Contains(user) Then
                Dim chat = Me._users(user)
                Dim message = args.Substring(user.Length + 1)
                chat.Chat(message)
                Me.TerminalWriteLine(String.Format(CultureInfo.CurrentCulture, Resources.SentTo, chat.User.Address))
            Else
                Me.TerminalWriteLine(Resources.ErrorNoUser)
            End If
        End Sub

        Private Sub ShowUsers()
            If Me.proxy IsNot Nothing Then
                Dim chatUsers = Me.proxy.GetUsers()
                Me._users.Clear()
                For Each cu As ChatUser In chatUsers
                    ' Don't add self
                    If New Uri(cu.Address) <> Me.localAddress Then Me._users.Add(New PeerChat(Me.userName, Me.localAddress.ToString(), cu))
                Next cu

                Me.TerminalWriteLine(Resources.UsersTitle)
                Me.TerminalWriteLine(Resources.Line)
                For Each pc As PeerChat In Me._users
                    Me.TerminalWriteLine(pc.User.Username)
                Next pc
            End If
        End Sub

        Private Sub QuitApp()
            Me.TerminalWriteLine(Resources.Goodbye)
            System.Threading.Thread.Sleep(500)
            Application.Exit()
        End Sub

        Private Sub SetAppState(ByVal state As TerminalState)
            Me.appState = state
            Select Case Me.appState
                Case TerminalState.Command
                    Me.prompt = ">"
                Case TerminalState.Login
                    Me.prompt = "Username >"
                Case TerminalState.Chat
                    Me.prompt = String.Format(CultureInfo.CurrentCulture, "chat {0} >", Me.userName)
            End Select
        End Sub

        Private Sub ProcessLogin()
            Dim input = Me.TerminalReadLine()
            If String.IsNullOrEmpty(input) Then
                Me.TerminalWriteLine(Resources.UsernameBackOrQuit)
            Else
                Dim args As String
                Dim cmd = ParseCmd(input, args)
                Select Case cmd.ToUpperInvariant()
                    Case "BACK"
                        Me.SetAppState(TerminalState.Command)
                    Case "QUIT"
                        Me.QuitApp()
                    Case Else
                        Me.userName = cmd
                        Me.TerminalWriteLine(Resources.OpeningChat)
                        Me.GoOnline()
                        Me.TerminalWriteLine(Resources.ChatAvailable)
                        Me.SetAppState(TerminalState.Chat)
                        Exit Select
                End Select
            End If

            Me.ShowPrompt()
        End Sub

        Private Sub ProcessCommand()
            Dim args As String
            Dim input = Me.TerminalReadLine()
            Dim cmd As String = ParseCmd(input, args)
            If (Not String.IsNullOrEmpty(cmd)) Then
                Select Case cmd.ToUpperInvariant()
                    Case "LOGIN"
                        Me.SetAppState(TerminalState.Login)
                    Case "QUIT"
                        Me.TerminalWrite(Resources.Goodbye)
                        Application.Exit()
                    Case Else
                        Me.TerminalWriteLine(Resources.LoginOrQuit)
                End Select
            End If

            Me.ShowPrompt()
        End Sub

        Private Function TerminalReadLine() As String
            Dim cmd = textBoxTerminal.Text.Substring(Me.cmdIndex, textBoxTerminal.Text.Length - 2 - Me.cmdIndex)
            Return cmd
        End Function

        Private Shared Function ParseCmd(ByVal input As String, <System.Runtime.InteropServices.Out()> ByRef args As String) As String
            If String.IsNullOrEmpty(input) Then
                args = Nothing
                Return Nothing
            Else
                Dim split() = input.Split(" "c)
                If split.Length = 1 Then
                    args = Nothing
                Else
                    args = input.Substring(split(0).Length + 1)
                End If

                Return split(0)
            End If
        End Function

        Public Sub ProcessSimpleMessage(ByVal message As ChatMessage)
            Invoke(New EventHandler(Sub(o As Object, a As EventArgs)
                                        Me.AppendNewLine()
                                        Me.TerminalWriteLine(String.Format(CultureInfo.CurrentCulture, Resources.SayMessage, message.UserName, message.Message))
                                        Me.ShowPrompt()
                                    End Sub))
        End Sub

        Private Sub TextBoxTerminal_MouseClick() Handles textBoxTerminal.MouseClick
            Me.SetCaret()
        End Sub

        Private Sub TextBoxTerminal_MouseDoubleClick() Handles textBoxTerminal.MouseDoubleClick
            Me.SetCaret()
        End Sub

        Private Sub TextBoxTerminal_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs) Handles textBoxTerminal.KeyDown
            Select Case e.KeyCode
                Case Keys.Up
                    e.SuppressKeyPress = True
                Case Keys.Down
                    e.SuppressKeyPress = True
                Case Keys.Back
                    If Me.TerminalColumn = Me.prompt.Length + 1 Then e.SuppressKeyPress = True

                    Exit Select

                Case Keys.Left
                    If Me.TerminalColumn = Me.prompt.Length + 1 Then e.SuppressKeyPress = True

                    Exit Select
            End Select
        End Sub
    End Class

    Public Enum TerminalState
        Command
        Login
        Chat
    End Enum
End Namespace