// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace Microsoft.Samples.Discovery.LegacyChat
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.ServiceModel;
    using System.Windows.Forms;
    using DiscoveryProxy.Contracts;
    using Microsoft.Samples.Discovery.Contracts;
    using Microsoft.Samples.Discovery.LegacyChat.Properties;

    public partial class LegacyChatForm : Form
    {
        private int cmdIndex;
        private TerminalState appState;
        private string prompt = ">";
        private string userName;
        private UserCollection users = new UserCollection();
        private ILegacyChatProxy proxy;
        private Uri localAddress;
        private ServiceHost chatServiceHost;

        public LegacyChatForm()
        {
            InitializeComponent();
            this.AppendNewLine();
            this.ShowPrompt();
        }

        private void OpenChatService()
        {
            this.localAddress = new Uri("http://localhost:8000/" + Guid.NewGuid().ToString());

            // Create a singleton instance for the host
            this.chatServiceHost = new ServiceHost(new ChatService(this), this.localAddress);
            this.chatServiceHost.AddServiceEndpoint(
                typeof(ISimpleChatService), 
                new BasicHttpBinding(), 
                this.localAddress);
            this.chatServiceHost.Open();
        }

        private void CloseChatService()
        {
            this.chatServiceHost.BeginClose(
                TimeSpan.FromSeconds(5),
                (result) => { chatServiceHost.EndClose(result); },
                null);
        }

        internal UserCollection Users
        {
            get
            {
                return this.users;
            }
        }

        private void ShowPrompt()
        {
            this.TerminalWrite(this.prompt);
        }

        private void AppendNewLine()
        {
            textBoxTerminal.Text += Environment.NewLine;
        }

        private void TerminalWriteLine(string text)
        {
            textBoxTerminal.Text += text + Environment.NewLine;
            this.SetCaret();
        }

        private void TerminalWrite(string text)
        {
            textBoxTerminal.Text += text;
            this.SetCaret();
        }

        private void SetCaret()
        {
            this.cmdIndex = textBoxTerminal.Text.Length;
            textBoxTerminal.SelectionStart = textBoxTerminal.Text.Length;
            textBoxTerminal.ScrollToCaret();
        }

        private void TextBoxTerminal_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Enter:
                    {
                        switch (this.appState)
                        {
                            case TerminalState.Command:
                                this.ProcessCommand();
                                break;
                            case TerminalState.Login:
                                this.ProcessLogin();
                                break;
                            case TerminalState.Chat:
                                this.ProcessChat();
                                break;
                        }

                        break;
                    }

                case Keys.Up:
                    e.SuppressKeyPress = true;
                    break;
                case Keys.Down:
                    e.SuppressKeyPress = true;
                    break;
            }
        }

        private int TerminalLine
        {
            get
            {
                return textBoxTerminal.GetLineFromCharIndex(textBoxTerminal.SelectionStart);
            }
        }

        private int TerminalColumn
        {
            get
            {
                return textBoxTerminal.SelectionStart - textBoxTerminal.GetFirstCharIndexOfCurrentLine() + 1;
            }
        }

        private void ProcessChat()
        {
            string input = this.TerminalReadLine();
            string args;
            string cmd = ParseCmd(input, out args);
            if (!string.IsNullOrEmpty(cmd))
            {
                switch (cmd.ToUpperInvariant())
                {
                    case "BYE":
                        this.GoOffline();
                        break;
                    case "USERS":
                        this.ShowUsers();
                        break;
                    case "SAY":
                        this.SayTo(args);
                        break;
                    case "QUIT":
                        this.QuitApp();
                        break;
                    default:
                        this.TerminalWriteLine(Resources.UserQuit);
                        break;
                }
            }

            this.ShowPrompt();
        }

        private void GoOnline()
        {
            this.TerminalWriteLine(Resources.ChatOnlineAt);
            this.OpenChatService();
            if (this.proxy == null)
            {
                NetTcpBinding binding = new NetTcpBinding();
                EndpointAddress address = 
                    new EndpointAddress("net.tcp://localhost:8001/legacyproxy");
                this.proxy = ChannelFactory<ILegacyChatProxy>.CreateChannel(binding, address);
            }

            this.proxy.Register(this.localAddress, this.userName);
            this.TerminalWriteLine(
                string.Format(
                CultureInfo.CurrentCulture,
                Resources.ChatOnlineAt,
                this.localAddress.ToString()));
            this.ShowUsers();
        }

        private void GoOffline()
        {
            this.TerminalWriteLine(Resources.GoingOffline);
            if (this.proxy != null)
            {
                this.proxy.Unregister(this.localAddress);
            }

            this.CloseChatService();
            this.TerminalWriteLine(Resources.LoggedOut);
            this.SetAppState(TerminalState.Command);
        }

        private void SayTo(string args)
        {
            string[] split = args.Split(' ');
            if (split.Length < 2)
            {
                this.TerminalWriteLine(Resources.ErrorNoMessage);
                return;
            }

            string user = split[0];
            if (this.users.Contains(user))
            {
                PeerChat chat = this.users[user];
                string message = args.Substring(user.Length + 1);
                chat.Chat(message);
                this.TerminalWriteLine(
                    string.Format(
                    CultureInfo.CurrentCulture,
                    Resources.SentTo,
                    chat.User.Address));
            }
            else
            {
                this.TerminalWriteLine(Resources.ErrorNoUser);
            }
        }

        private void ShowUsers()
        {
            if (this.proxy != null)
            {
                List<ChatUser> chatUsers = this.proxy.GetUsers();
                this.users.Clear();
                foreach (ChatUser cu in chatUsers)
                {
                    // Don't add self
                    if (new Uri(cu.Address) != this.localAddress)
                    {
                        this.users.Add(new PeerChat(
                            this.userName, this.localAddress.ToString(), cu));
                    }
                }

                this.TerminalWriteLine(Resources.UsersTitle);
                this.TerminalWriteLine(Resources.Line);
                foreach (PeerChat pc in this.users)
                {
                    this.TerminalWriteLine(pc.User.Username);
                }
            }
        }

        private void QuitApp()
        {
            this.TerminalWriteLine(Resources.Goodbye);
            System.Threading.Thread.Sleep(500);
            Application.Exit();
        }

        private void SetAppState(TerminalState state)
        {
            this.appState = state;
            switch (this.appState)
            {
                case TerminalState.Command:
                    this.prompt = ">";
                    break;
                case TerminalState.Login:
                    this.prompt = "Username >";
                    break;
                case TerminalState.Chat:
                    this.prompt = string.Format(CultureInfo.CurrentCulture, "chat {0} >", this.userName);
                    break;
            }
        }

        private void ProcessLogin()
        {
            string input = this.TerminalReadLine();
            if (string.IsNullOrEmpty(input))
            {
                this.TerminalWriteLine(Resources.UsernameBackOrQuit);
            }
            else
            {
                string args;
                string cmd = ParseCmd(input, out args);
                switch (cmd.ToUpperInvariant())
                {
                    case "BACK":
                        this.SetAppState(TerminalState.Command);
                        break;
                    case "QUIT":
                        this.QuitApp();
                        break;
                    default:
                        {
                            this.userName = cmd;
                            this.TerminalWriteLine(Resources.OpeningChat);
                            this.GoOnline();
                            this.TerminalWriteLine(Resources.ChatAvailable);
                            this.SetAppState(TerminalState.Chat);
                            break;
                        }
                }
            }

            this.ShowPrompt();
        }

        private void ProcessCommand()
        {
            string args;
            string input = this.TerminalReadLine();
            string cmd = ParseCmd(input, out args);
            if (!string.IsNullOrEmpty(cmd))
            {
                switch (cmd.ToUpperInvariant())
                {
                    case "LOGIN":
                        this.SetAppState(TerminalState.Login);
                        break;
                    case "QUIT":
                        this.TerminalWrite(Resources.Goodbye);
                        Application.Exit();
                        break;
                    default:
                        this.TerminalWriteLine(Resources.LoginOrQuit);
                        break;
                }
            }

            this.ShowPrompt();
        }

        private string TerminalReadLine()
        {
            string cmd = textBoxTerminal.Text.Substring(
                this.cmdIndex,
                textBoxTerminal.Text.Length - 2 - this.cmdIndex);
            return cmd;
        }

        private static string ParseCmd(string input, out string args)
        {
            if (string.IsNullOrEmpty(input))
            {
                args = null;
                return null;
            }
            else
            {
                string[] split = input.Split(' ');
                if (split.Length == 1)
                {
                    args = null;
                }
                else
                {
                    args = input.Substring(split[0].Length + 1);
                }

                return split[0];
            }
        }

        public void ProcessSimpleMessage(ChatMessage message)
        {
            Invoke(new EventHandler(delegate(object o, EventArgs a)
                {
                    this.AppendNewLine();
                    this.TerminalWriteLine(
                        string.Format(
                        CultureInfo.CurrentCulture,
                        Resources.SayMessage,
                        message.UserName,
                        message.Message));
                    this.ShowPrompt();
                }));
        }

        private void TextBoxTerminal_MouseClick(object sender, MouseEventArgs e)
        {
            this.SetCaret();
        }

        private void TextBoxTerminal_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.SetCaret();
        }

        private void TextBoxTerminal_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Up:
                    e.SuppressKeyPress = true;
                    break;
                case Keys.Down:
                    e.SuppressKeyPress = true;
                    break;
                case Keys.Back:
                    {
                        if (this.TerminalColumn == this.prompt.Length + 1)
                        {
                            e.SuppressKeyPress = true;
                        }

                        break;
                    }

                case Keys.Left:
                    {
                        if (this.TerminalColumn == this.prompt.Length + 1)
                        {
                            e.SuppressKeyPress = true;
                        }

                        break;
                    }
            }
        }
    }

    public enum TerminalState
    {
        Command,
        Login,
        Chat
    }
}